<?php
/*
Plugin Name: WP2Bear
Description: Export WordPress posts to Markdown format compatible with bearblog.dev
Donate link: https://www.paypal.me/donateKulkaNicolas
Version: 0.1.0
Author: mgx.me
Author URI: https://mgx.me/
License: GPLv2 or later
License URI: https://www.gnu.org/licenses/gpl-2.0.html
Requires at least: 6.1
Requires PHP: 7.4
Tested up to: 6.7.1
*/

class WP2Bear_Plugin
{
    private $options;

    public function __construct()
    {
        add_action('admin_menu', [$this, 'add_export_page']);
        add_action('admin_init', [$this, 'register_settings']);
        add_action('admin_post_export_markdown', [$this, 'export_markdown']);
        add_action('admin_post_download_all_posts', [$this, 'download_all_posts']);

        // Load plugin settings
        $this->options = get_option('markdown_export_settings', [
            'fallback_meta_description' => '',
            'fallback_meta_image' => '',
            'make_discoverable' => 'true',
            'class_name' => 'wp',
            'tag_source' => 'tags',
            'override_meta_image_domain' => '',
            'set_alias_prefix' => '',
            'override_canonical_domain' => '',
            'convert_to_markdown' => 'true',
            'lang' => 'en',
            'include_canonical' => 'false',
            'include_alias' => 'false'
        ]);
    }


    public function add_export_page()
    {
        add_management_page(
            'WP2Bear',
            'WP2Bear',
            'manage_options',
            'wp2bear',
            [$this, 'render_export_page']
        );
    }

    public function register_settings()
    {
        register_setting(
            'markdown_export_settings_group',
            'markdown_export_settings',
            [
                'sanitize_callback' => [$this, 'sanitize_settings'],
            ]
        );
    }

    public function sanitize_settings($input)
    {
        $output = [];
        $output['fallback_meta_description'] = sanitize_text_field($input['fallback_meta_description']);
        $output['fallback_meta_image'] = esc_url_raw($input['fallback_meta_image']);
        $output['make_discoverable'] = in_array($input['make_discoverable'], ['true', 'false']) ? $input['make_discoverable'] : 'true';
        $output['class_name'] = sanitize_text_field($input['class_name']);
        $output['tag_source'] = in_array($input['tag_source'], ['tags', 'categories', 'both']) ? $input['tag_source'] : 'tags';
        $output['override_meta_image_domain'] = esc_url_raw($input['override_meta_image_domain']);
        $output['set_alias_prefix'] = sanitize_text_field($input['set_alias_prefix']);
        $output['override_canonical_domain'] = esc_url_raw($input['override_canonical_domain']);
        $output['convert_to_markdown'] = in_array($input['convert_to_markdown'], ['true', 'false']) ? $input['convert_to_markdown'] : 'true';
        $output['lang'] = sanitize_text_field($input['lang']);
        $output['include_canonical'] = in_array($input['include_canonical'], ['true', 'false']) ? $input['include_canonical'] : 'false';
        $output['include_alias'] = in_array($input['include_alias'], ['true', 'false']) ? $input['include_alias'] : 'false';
        return $output;
    }


    public function render_export_page()
    {
        ?>
        <div class="wrap">
            <style>
                .wrap {
                    background: #fdfdfd;
                    padding: 10px 20px;
                    border-radius: 10px
                }

                .wrap h1 {
                    font-weight: 600 !important
                }

                .wp-core-ui .notice.is-dismissible {
                    background: #fff0;
                    color: inherit
                }

                tr {
                    background-color: #F0F0F1;
                    display: block;
                    padding: 0 10px;
                    margin-bottom: 10px;
                    border-radius: 5px
                }

                @media (prefers-color-scheme:dark) {
                    tr {
                        background: #01242e;
                        border: 1px solid #013240
                    }

                    .wrap {
                        background: #01242e;
                        color: #fff
                    }

                    .wrap h1,
                    .wrap th {
                        color: #fff
                    }

                    .wp-core-ui select {
                        color: #fff;
                        border-color: #033e4f;
                        background: #01242e url(data:image/svg+xml;charset=US-ASCII,%3Csvg%20width%3D%2220%22%20height%3D%2220%22%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%3E%3Cpath%20d%3D%22M5%206l5%205%205-5%202%201-7%207-7-7%202-1z%22%20fill%3D%22%23555%22%2F%3E%3C%2Fsvg%3E) no-repeat right 5px top 55%
                    }

                    .wp-core-ui select:hover,
                    .wp-core-ui select:select {
                        color: whitesmoke !important
                    }

                    input[type="color"],
                    input[type="date"],
                    input[type="datetime-local"],
                    input[type="datetime"],
                    input[type="email"],
                    input[type="month"],
                    input[type="number"],
                    input[type="password"],
                    input[type="search"],
                    input[type="tel"],
                    input[type="text"],
                    input[type="time"],
                    input[type="url"],
                    input[type="week"],
                    select,
                    textarea {
                        border: 1px solid #033e4f;
                        background-color: #01242e;
                        color: #fff
                    }
                }

                .wp2bear-h1:after {
                    content: 'v0.1.0';
                    margin-left: 10px;
                    opacity: .5;
                }
            </style>
            <h1 class="wp2bear-h1">WP2Bear Settings</h1>
            <p>WP2Bear (beta) allows you to export your WordPress posts into Markdown format, compatible with bearblog.dev. The
                plugin ensures that all the necessary metadata (e.g., title, description, image, etc.) is properly formatted for
                seamless integration with Bearblog. It also provides options to customize certain metadata, including the
                ability to override values for meta images, aliases, and canonical URLs, making it more flexible when exporting
                your content. <a target="_blank" href="https://mgx.me/wp2bear-wordpress-plugin">More info & usage tips</a>.</p>
            <form method="post" action="options.php">
                <?php
                settings_fields('markdown_export_settings_group');
                do_settings_sections('markdown_export_settings_group');
                ?>
                <table class="form-table">
                    <tr valign="top">
                        <th scope="row">Convert Post Content</th>
                        <td>
                            <select name="markdown_export_settings[convert_to_markdown]">
                                <option value="true" <?php selected($this->options['convert_to_markdown'], 'true'); ?>>Yes
                                </option>
                                <option value="false" <?php selected($this->options['convert_to_markdown'], 'false'); ?>>No
                                </option>
                            </select>
                            <p style="font-size:12px;opacity:.75;">Choose whether to convert post content to Markdown format or
                                keep it as
                                WP default (HTML).</p>
                        </td>
                    </tr>
                    <tr valign="top">
                        <th scope="row">Language Code</th>
                        <td>
                            <input type="text" name="markdown_export_settings[lang]"
                                value="<?php echo esc_attr($this->options['lang']); ?>" class="regular-text" placeholder="en">
                            <p style="font-size:12px;opacity:.75;">Enter the language code for your blog (e.g., "en" for
                                English). Defaults to en. <a target="_blank"
                                    href="https://docs.bearblog.dev/language-cheatsheet?ref=wp2bear">Check the language
                                    cheatsheet</a></p>
                        </td>
                    </tr>
                    <tr valign="top">
                        <th scope="row">Fallback Meta Description</th>
                        <td>
                            <input type="text" name="markdown_export_settings[fallback_meta_description]"
                                value="<?php echo esc_attr($this->options['fallback_meta_description']); ?>"
                                class="regular-text" placeholder="Bears are pretty cool, according to science.">
                            <p style="font-size:12px;opacity:.75;">Optional. Specify a default meta description if none is
                                available.</p>
                        </td>
                    </tr>
                    <tr valign="top">
                        <th scope="row">Fallback Meta Image</th>
                        <td>
                            <input type="text" name="markdown_export_settings[fallback_meta_image]"
                                value="<?php echo esc_attr($this->options['fallback_meta_image']); ?>" class="regular-text"
                                placeholder="https://i.imgur.com/3jxqrKP.jpeg">
                            <p style="font-size:12px;opacity:.75;">Optional. Define a default image for posts without featured
                                images.</p>
                        </td>
                    </tr>
                    <tr valign="top">
                        <th scope="row">Make Discoverable</th>
                        <td>
                            <select name="markdown_export_settings[make_discoverable]">
                                <option value="true" <?php selected($this->options['make_discoverable'], 'true'); ?>>True
                                </option>
                                <option value="false" <?php selected($this->options['make_discoverable'], 'false'); ?>>False
                                </option>
                            </select>
                            <p style="font-size:12px;opacity:.75;">Enable or disable discoverability on Bearblog.</p>
                        </td>
                    </tr>
                    <tr valign="top">
                        <th scope="row">CSS Class Name</th>
                        <td>
                            <input type="text" name="markdown_export_settings[class_name]"
                                value="<?php echo esc_attr($this->options['class_name']); ?>" placeholder="wp">
                            <p style="font-size:12px;opacity:.75;">Optional. Assign a class name for all your WP content.</p>
                        </td>
                    </tr>
                    <tr valign="top">
                        <th scope="row">Tag Source</th>
                        <td>
                            <select name="markdown_export_settings[tag_source]">
                                <option value="tags" <?php selected($this->options['tag_source'], 'tags'); ?>>WordPress Tags
                                </option>
                                <option value="categories" <?php selected($this->options['tag_source'], 'categories'); ?>>
                                    WordPress Categories</option>
                                <option value="both" <?php selected($this->options['tag_source'], 'both'); ?>>Combine Tags and
                                    Categories</option>
                            </select>
                            <p style="font-size:12px;opacity:.75;">Choose whether to export WordPress tags, categories, or both.
                            </p>
                        </td>
                    </tr>
                    <tr valign="top">
                        <th scope="row">Include Alias</th>
                        <td>
                            <select name="markdown_export_settings[include_alias]">
                                <option value="true" <?php selected($this->options['include_alias'], 'true'); ?>>Yes</option>
                                <option value="false" <?php selected($this->options['include_alias'], 'false'); ?>>No</option>
                            </select>
                            <p style="font-size:12px;opacity:.75;">Choose whether to include the alias in the exported Markdown
                                files.</p>
                        </td>
                    </tr>
                    <tr valign="top">
                        <th scope="row">Set Alias Prefix</th>
                        <td>
                            <input type="text" name="markdown_export_settings[set_alias_prefix]"
                                value="<?php echo esc_attr($this->options['set_alias_prefix']); ?>" class="regular-text"
                                placeholder="posts/">
                            <p style="font-size:12px;opacity:.75;">Alias prefix is useful if you have a unique permalink
                                structure in WP and want to carry it over to Bear.</p>
                        </td>
                    </tr>
                    <tr valign="top">
                        <th scope="row">Include Canonical URL</th>
                        <td>
                            <select name="markdown_export_settings[include_canonical]">
                                <option value="true" <?php selected($this->options['include_canonical'], 'true'); ?>>Yes
                                </option>
                                <option value="false" <?php selected($this->options['include_canonical'], 'false'); ?>>No
                                </option>
                            </select>
                            <p style="font-size:12px;opacity:.75;">Choose whether to include the canonical URL in the exported
                                Markdown files.</p>
                        </td>
                    </tr>
                    <tr valign="top">
                        <th scope="row">Override Canonical Domain</th>
                        <td>
                            <input type="text" name="markdown_export_settings[override_canonical_domain]"
                                value="<?php echo esc_attr($this->options['override_canonical_domain']); ?>"
                                class="regular-text" placeholder="https://old.example.com/">
                            <p style="font-size:12px;opacity:.75;">Ensure search engines are directed to the correct domain.</p>
                        </td>
                    </tr>
                    <tr valign="top">
                        <th scope="row">Override Image Domain</th>
                        <td>
                            <input type="text" name="markdown_export_settings[override_meta_image_domain]"
                                value="<?php echo esc_attr($this->options['override_meta_image_domain']); ?>"
                                class="regular-text" placeholder="https://static.example.com/">
                            <p style="font-size:12px;opacity:.75;">Optional. Helps adjust image URLs in exported posts. If
                                you've migrated your WordPress uploads (media) to a file hosting provider, this option makes
                                sure your images load from the correct source. Defaults to your current Wordpress media URLs.
                            </p>
                        </td>
                    </tr>
                </table>
                <div style="display: flex; justify-content: flex-start; align-items: center;">
                    <input style="margin-right: 10px;" type="submit" name="submit" id="submit" class="button button-primary"
                        value="Save Settings">
                    <input style="margin-right: 10px;" type="submit" form="export-form" class="button button-secondary"
                        value="Export Published Posts">
                    <input type="submit" form="download-all-form" class="button button-secondary" value="Export All Posts">
                </div>
            </form>

            <form id="export-form" method="post" action="<?php echo esc_url(admin_url('admin-post.php')); ?>"
                style="display: inline;">
                <input type="hidden" name="action" value="export_markdown">
                <?php wp_nonce_field('markdown_export_nonce'); ?>
            </form>

            <form id="download-all-form" method="post" action="<?php echo esc_url(admin_url('admin-post.php')); ?>"
                style="display: inline;">
                <input type="hidden" name="action" value="download_all_posts">
                <?php wp_nonce_field('wp2bear_download_all_nonce'); ?>
            </form>
            <p><strong>Note</strong>: WP2Bear (beta) is provided as is, without any express or implied warranties. Use at
                your own risk. The plugin's author is not responsible for any issues, including but not limited to: Loss of
                data, Plugin incompatibilities, Incorrect metadata export, Any other unintended behavior. By using this plugin,
                you acknowledge that you assume all responsibility for its use and consequences. If you remix, modify, or
                redistribute WP2Bear in any way, you must provide credit to <a href="https://mgx.me">mgx.me</a> in your
                project's documentation or credits section.</p>
        </div>
        <?php
    }

    public function export_markdown()
    {
        // Verify nonce and user capabilities
        check_admin_referer('markdown_export_nonce');
        if (!current_user_can('manage_options')) {
            wp_die('Unauthorized access');
        }

        // Initialize WP_Filesystem
        global $wp_filesystem;
        if (!function_exists('WP_Filesystem')) {
            require_once ABSPATH . 'wp-admin/includes/file.php';
        }
        WP_Filesystem();

        // Create temporary directory for export
        $upload_dir = wp_upload_dir();
        $export_dir = $upload_dir['basedir'] . '/wp2bear';
        if (!$wp_filesystem->exists($export_dir)) {
            if (!$wp_filesystem->mkdir($export_dir, 0755)) {
                wp_die('Failed to create export directory.');
            }
        }

        // Query published posts and pages
        $post_types = ['post', 'page'];
        $posts_query = new WP_Query([
            'post_type' => $post_types,
            'post_status' => 'publish',
            'posts_per_page' => -1
        ]);

        // Temporary files to track
        $markdown_files = [];

        // Process each post
        while ($posts_query->have_posts()) {
            $posts_query->the_post();
            $post = get_post();

            // Get Yoast meta if available
            $yoast_meta = $this->get_yoast_meta($post->ID);

            // Prepare Markdown front matter
            $front_matter = $this->prepare_front_matter($post, $yoast_meta);

            // Create Markdown file
            $filename = sanitize_file_name($post->post_name . '.md');
            $filepath = $export_dir . '/' . $filename;

            // Get post content and process it
            $content = $post->post_content;
            $content = $this->replace_image_urls($content);

            // Convert to Markdown if enabled
            if ($this->options['convert_to_markdown'] === 'true') {
                $content = $this->html_to_markdown($content);
            } else {
                // Still sanitize HTML even if not converting to Markdown
                $content = $this->sanitize_html($content);
            }

            // Write file
            $file_content = $front_matter . "___\n" . $content;
            if (!$wp_filesystem->put_contents($filepath, $file_content, 0644)) {
                wp_die('Failed to write Markdown file: ' . esc_html($filepath));
            }
            $markdown_files[] = $filepath;
        }

        // Reset post data
        wp_reset_postdata();

        // Create ZIP file
        $zip_filename = 'wp2bear-export-' . gmdate('Y-m-d') . '.zip';
        $zip_filepath = $upload_dir['basedir'] . '/' . $zip_filename;

        $zip = new ZipArchive();
        if ($zip->open($zip_filepath, ZipArchive::CREATE | ZipArchive::OVERWRITE) === TRUE) {
            foreach ($markdown_files as $file) {
                $zip->addFile($file, basename($file));
            }
            $zip->close();
        }

        // Trigger download using WP_Filesystem
        if ($wp_filesystem->exists($zip_filepath)) {
            header('Content-Type: application/zip');
            header('Content-Disposition: attachment; filename="' . esc_attr($zip_filename) . '"');
            header('Content-Length: ' . esc_attr($wp_filesystem->size($zip_filepath)));
            echo $wp_filesystem->get_contents($zip_filepath);
        } else {
            wp_die('Failed to create ZIP file.');
        }

        // Clean up
        foreach ($markdown_files as $file) {
            wp_delete_file($file);
        }
        wp_delete_file($zip_filepath);
        $wp_filesystem->rmdir($export_dir);

        exit();
    }

    public function download_all_posts()
    {
        // Verify nonce and user capabilities
        check_admin_referer('wp2bear_download_all_nonce');
        if (!current_user_can('manage_options')) {
            wp_die('Unauthorized access');
        }

        // Initialize WP_Filesystem
        global $wp_filesystem;
        if (!function_exists('WP_Filesystem')) {
            require_once ABSPATH . 'wp-admin/includes/file.php';
        }
        WP_Filesystem();

        // Create temporary directory for export
        $upload_dir = wp_upload_dir();
        $export_dir = $upload_dir['basedir'] . '/wp2bear';
        if (!$wp_filesystem->exists($export_dir)) {
            if (!$wp_filesystem->mkdir($export_dir, 0755)) {
                wp_die('Failed to create export directory.');
            }
        }

        // Query all posts and pages (including drafts, private, etc.)
        $post_types = ['post', 'page'];
        $posts_query = new WP_Query([
            'post_type' => $post_types,
            'post_status' => ['publish', 'draft', 'pending', 'private', 'future'],
            'posts_per_page' => -1
        ]);

        // Temporary files to track
        $markdown_files = [];

        // Process each post
        while ($posts_query->have_posts()) {
            $posts_query->the_post();
            $post = get_post();

            // Get Yoast meta if available
            $yoast_meta = $this->get_yoast_meta($post->ID);

            // Prepare Markdown front matter
            $front_matter = $this->prepare_front_matter($post, $yoast_meta);

            // Create Markdown file
            $filename = sanitize_file_name($post->post_name . '.md');
            $filepath = $export_dir . '/' . $filename;

            // Get post content and process it
            $content = $post->post_content;
            $content = $this->replace_image_urls($content);

            // Convert to Markdown if enabled
            if ($this->options['convert_to_markdown'] === 'true') {
                $content = $this->html_to_markdown($content);
            } else {
                // Still sanitize HTML even if not converting to Markdown
                $content = $this->sanitize_html($content);
            }

            // Write file
            $file_content = $front_matter . "___\n" . $content;
            if (!$wp_filesystem->put_contents($filepath, $file_content, 0644)) {
                wp_die('Failed to write Markdown file: ' . esc_html($filepath));
            }
            $markdown_files[] = $filepath;
        }

        // Reset post data
        wp_reset_postdata();

        // Create ZIP file
        $zip_filename = 'wp2bear-export-all-' . gmdate('Y-m-d') . '.zip';
        $zip_filepath = $upload_dir['basedir'] . '/' . $zip_filename;

        $zip = new ZipArchive();
        if ($zip->open($zip_filepath, ZipArchive::CREATE | ZipArchive::OVERWRITE) === TRUE) {
            foreach ($markdown_files as $file) {
                $zip->addFile($file, basename($file));
            }
            $zip->close();
        }

        // Trigger download using WP_Filesystem
        if ($wp_filesystem->exists($zip_filepath)) {
            header('Content-Type: application/zip');
            header('Content-Disposition: attachment; filename="' . esc_attr($zip_filename) . '"');
            header('Content-Length: ' . esc_attr($wp_filesystem->size($zip_filepath)));
            echo $wp_filesystem->get_contents($zip_filepath);
        } else {
            wp_die('Failed to create ZIP file.');
        }

        // Clean up
        foreach ($markdown_files as $file) {
            wp_delete_file($file);
        }
        wp_delete_file($zip_filepath);
        $wp_filesystem->rmdir($export_dir);

        exit();
    }


    private function html_to_markdown($content)
    {
        // Basic HTML sanitization first
        $content = $this->sanitize_html($content);

        // Convert common HTML elements to Markdown
        $content = preg_replace('/<h1[^>]*>(.*?)<\/h1>/i', '# $1', $content);
        $content = preg_replace('/<h2[^>]*>(.*?)<\/h2>/i', '## $1', $content);
        $content = preg_replace('/<h3[^>]*>(.*?)<\/h3>/i', '### $1', $content);
        $content = preg_replace('/<h4[^>]*>(.*?)<\/h4>/i', '#### $1', $content);
        $content = preg_replace('/<h5[^>]*>(.*?)<\/h5>/i', '##### $1', $content);
        $content = preg_replace('/<h6[^>]*>(.*?)<\/h6>/i', '###### $1', $content);

        // Convert links
        $content = preg_replace_callback(
            '/<a[^>]+href=["\'](.*?)["\'](.*?)>(.*?)<\/a>/i',
            function ($matches) {
                $url = $matches[1];
                $text = strip_tags($matches[3]);
                return "[$text]($url)";
            },
            $content
        );

        // Convert images with alt text
        $content = preg_replace_callback(
            '/<img[^>]+src=["\'](.*?)["\'](.*?)>/i',
            function ($matches) {
                $src = $matches[1];
                $alt = '';
                if (preg_match('/alt=["\'](.*?)["\']/i', $matches[2], $alt_matches)) {
                    $alt = $alt_matches[1];
                }
                return "![$alt]($src)";
            },
            $content
        );

        // Convert lists
        $content = preg_replace('/<ul[^>]*>(.*?)<\/ul>/is', '$1', $content);
        $content = preg_replace('/<ol[^>]*>(.*?)<\/ol>/is', '$1', $content);
        $content = preg_replace('/<li[^>]*>(.*?)<\/li>/i', '- $1', $content);

        // Convert emphasis
        $content = preg_replace('/<strong[^>]*>(.*?)<\/strong>/i', '**$1**', $content);
        $content = preg_replace('/<b[^>]*>(.*?)<\/b>/i', '**$1**', $content);
        $content = preg_replace('/<em[^>]*>(.*?)<\/em>/i', '*$1*', $content);
        $content = preg_replace('/<i[^>]*>(.*?)<\/i>/i', '*$1*', $content);

        // Convert blockquotes
        $content = preg_replace('/<blockquote[^>]*>(.*?)<\/blockquote>/is', '> $1', $content);

        // Convert code blocks
        $content = preg_replace('/<pre[^>]*><code[^>]*>(.*?)<\/code><\/pre>/is', '```$1```', $content);
        $content = preg_replace('/<code[^>]*>(.*?)<\/code>/i', '`$1`', $content);

        // Convert paragraphs
        $content = preg_replace('/<p[^>]*>(.*?)<\/p>/is', '$1', $content);

        // Convert tables
        $content = preg_replace_callback(
            '/<table[^>]*>(.*?)<\/table>/is',
            function ($matches) {
                $tableContent = $matches[1];
                $rows = [];
                $header = [];
                $isHeader = true;

                // Extract rows
                preg_match_all('/<tr[^>]*>(.*?)<\/tr>/is', $tableContent, $rowMatches);
                foreach ($rowMatches[1] as $row) {
                    $cells = [];
                    preg_match_all('/<(th|td)[^>]*>(.*?)<\/(th|td)>/is', $row, $cellMatches);
                    foreach ($cellMatches[2] as $cell) {
                        $cells[] = trim(strip_tags($cell));
                    }
                    if ($isHeader) {
                        $header = $cells;
                        $isHeader = false;
                    } else {
                        $rows[] = $cells;
                    }
                }

                // Build Markdown table
                $markdownTable = '';
                if (!empty($header)) {
                    $markdownTable .= '| ' . implode(' | ', $header) . " |\n";
                    $markdownTable .= '|' . str_repeat(' --- |', count($header)) . "\n";
                }
                foreach ($rows as $row) {
                    $markdownTable .= '| ' . implode(' | ', $row) . " |\n";
                }
                return $markdownTable;
            },
            $content
        );

        // Clean up
        $content = strip_tags($content);
        $content = html_entity_decode($content);
        $content = str_replace('&nbsp;', ' ', $content);
        $content = preg_replace('/\n{3,}/', "\n\n", $content);
        $content = trim($content);

        return $content;
    }


    private function sanitize_html($content)
    {
        // Remove potentially harmful HTML elements and their contents
        $harmful_elements = array(
            'script',
            'style',
            'iframe',
            'frame',
            'frameset',
            'object',
            'embed',
            'applet',
            'form',
            'input',
            'textarea',
            'button',
            'marquee',
            'base',
            'meta',
            'link'
        );

        foreach ($harmful_elements as $element) {
            $content = preg_replace('/<' . $element . '[^>]*>.*?<\/' . $element . '>/is', '', $content);
            $content = preg_replace('/<' . $element . '[^>]*\/>/i', '', $content);
        }

        // Remove potentially harmful attributes
        $harmful_attributes = array(
            'onabort',
            'onblur',
            'onchange',
            'onclick',
            'ondblclick',
            'onerror',
            'onfocus',
            'onkeydown',
            'onkeypress',
            'onkeyup',
            'onload',
            'onmousedown',
            'onmousemove',
            'onmouseout',
            'onmouseover',
            'onmouseup',
            'onreset',
            'onresize',
            'onselect',
            'onsubmit',
            'onunload',
            'javascript',
            'expression',
            'behavior',
            'vbscript',
            'mocha',
            'livescript'
        );

        foreach ($harmful_attributes as $attribute) {
            $content = preg_replace('/\s*' . $attribute . '\s*=\s*["\'][^"\']*["\']/i', '', $content);
        }

        // Remove comments
        $content = preg_replace('/<!--.*?-->/s', '', $content);

        return $content;
    }

    private function replace_image_urls($content)
    {
        $override_meta_image_domain = $this->options['override_meta_image_domain'];
        if (!empty($override_meta_image_domain)) {
            $pattern = '/<img[^>]+src=["\'](https?:\/\/(?:www\.)?[^"\']+)["\']/i';
            $replacement = function ($matches) use ($override_meta_image_domain) {
                $url = $matches[1];
                $parsed_url = wp_parse_url($url);
                if (isset($parsed_url['host'])) {
                    $new_url = rtrim($override_meta_image_domain, '/') . $parsed_url['path'];
                    if (isset($parsed_url['query'])) {
                        $new_url .= '?' . $parsed_url['query'];
                    }
                    if (isset($parsed_url['fragment'])) {
                        $new_url .= '#' . $parsed_url['fragment'];
                    }
                    return '<img src="' . esc_url($new_url) . '"';
                }
                return $matches[0];
            };
            $content = preg_replace_callback($pattern, $replacement, $content);
        }
        return $content;
    }

    private function get_yoast_meta($post_id)
    {
        if (!is_plugin_active('wordpress-seo/wp-seo.php')) {
            return [];
        }
        return [
            'meta_description' => get_post_meta($post_id, '_yoast_wpseo_metadesc', true),
            'canonical_url' => get_post_meta($post_id, '_yoast_wpseo_canonical', true)
        ];
    }

    private function prepare_front_matter($post, $yoast_meta)
    {
        $tags = [];
        switch ($this->options['tag_source']) {
            case 'tags':
                $post_tags = get_the_tags($post->ID);
                if ($post_tags) {
                    $tags = wp_list_pluck($post_tags, 'name');
                }
                break;
            case 'categories':
                $post_categories = get_the_category($post->ID);
                if ($post_categories) {
                    $tags = wp_list_pluck($post_categories, 'name');
                }
                break;
            case 'both':
            default:
                $post_categories = get_the_category($post->ID);
                $post_tags = get_the_tags($post->ID);
                if ($post_categories) {
                    $tags = array_merge($tags, wp_list_pluck($post_categories, 'name'));
                }
                if ($post_tags) {
                    $tags = array_merge($tags, wp_list_pluck($post_tags, 'name'));
                }
                break;
        }

        $featured_image = '';
        if (has_post_thumbnail($post->ID)) {
            $featured_image = wp_get_attachment_image_url(get_post_thumbnail_id($post->ID), 'full');
            // Apply domain override only to the featured image
            if (!empty($this->options['override_meta_image_domain'])) {
                $featured_image = $this->override_domain($featured_image, $this->options['override_meta_image_domain']);
            }
        }

        // Use fallback image if no featured image is available
        $meta_image_url = $featured_image;
        if (empty($meta_image_url) && !empty($this->options['fallback_meta_image'])) {
            $meta_image_url = $this->options['fallback_meta_image'];
        }

        $meta_description = '';
        if (!empty($yoast_meta['meta_description'])) {
            $meta_description = $yoast_meta['meta_description'];
        } elseif (!empty($post->post_excerpt)) {
            $meta_description = wp_trim_words($post->post_excerpt, 30);
        } elseif (!empty($this->options['fallback_meta_description'])) {
            $meta_description = $this->options['fallback_meta_description'];
        } else {
            $meta_description = wp_trim_words($post->post_content, 30);
        }

        $front_matter = "title: " . esc_html($post->post_title) . "\n" .
            "link: " . esc_html($post->post_name) . "\n";

        if ($this->options['include_alias'] === 'true') {
            $alias_url = '/' . esc_html($post->post_name);
            if (!empty($this->options['set_alias_prefix'])) {
                $alias_url = $this->override_domain($alias_url, $this->options['set_alias_prefix']);
            }
            $front_matter .= "alias: " . esc_html($alias_url) . "\n";
        }

        if ($this->options['include_canonical'] === 'true') {
            $canonical_url = !empty($yoast_meta['canonical_url'])
                ? $yoast_meta['canonical_url']
                : get_permalink($post->ID);
            if (!empty($this->options['override_canonical_domain'])) {
                $canonical_url = $this->override_domain($canonical_url, $this->options['override_canonical_domain']);
            }
            $front_matter .= "canonical_url: " . esc_url($canonical_url) . "\n";
        }

        $front_matter .= "published_date: " . esc_html(get_the_date('Y-m-d', $post->ID)) . "\n" .
            "is_page: " . ($post->post_type === 'page' ? 'true' : 'false') . "\n" .
            "meta_description: " . esc_html($meta_description) . "\n" .
            "meta_image: " . esc_url($meta_image_url) . "\n" .
            "tags: " . esc_html(implode(', ', array_unique($tags))) . "\n" .
            "make_discoverable: " . esc_html($this->options['make_discoverable']) . "\n" .
            "class_name: " . esc_html($this->options['class_name']) . "\n" .
            "lang: " . esc_html($this->options['lang']) . "\n";

        return $front_matter;
    }

    private function override_domain($url, $new_domain)
    {
        $parsed_url = wp_parse_url($url);
        if ($parsed_url === false) {
            return $url;
        }
        $path = $parsed_url['path'] ?? '';
        $query = isset($parsed_url['query']) ? '?' . $parsed_url['query'] : '';
        $fragment = isset($parsed_url['fragment']) ? '#' . $parsed_url['fragment'] : '';
        return rtrim($new_domain, '/') . '/' . ltrim($path, '/') . $query . $fragment;
    }
}

new WP2Bear_Plugin();